#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXNODE 1000000
#define MAXPOW	1000000000
#define BUFLEN 1024

int capacity[MAXNODE+1] = {0};
int power[MAXNODE+1] = {0};
int parent[MAXNODE+1] = {0};
int source[MAXNODE+1] = {0};

char buf[BUFLEN];

int N;

static void fail(void) {
  printf("Nelze\n");
  exit(0);
}

static int get_source(int me) {
  return source[me] == me ? me : get_source(source[me]);
}

int walk_tree(int me) {
  if (power[me] > 0) { // power station
    source[me] = me;
    for (int i=me+1;i<=N;i++) { // find all the subtrees
      if (parent[i] == me) {
	int got = walk_tree(i);
	if (got < 0) { // there is a town-like subtree
	  if (capacity[i] + got < 0) // not enough capacity
	    fail();

	  power[me] += got;
	  source[i] = me;
	}
      }

      if (power[me] < 0) // we cannot take all the subtrees
	fail();
    }
    return power[me];
  } else if (power[me] < 0) { // town
    int maxtown = 0;
    int m = 0;
    for (int i=me+1;i<=N;i++) { // find all the subtrees
      if (parent[i] == me) {
	int got = walk_tree(i);
	if (got < 0) { // there is a town-like subtree
	  if (capacity[i] + got < 0) // not enough capacity
	    fail();

	  power[me] += got;
	  source[i] = me;
	} else if (got > 0) { // a power-station-like subtree
	  if (capacity[i] < got)
	    got = capacity[i];

	  if (maxtown < got) { // looking for the best
	    maxtown = got;
	    m = i;
	  }
	}
      }
    }

    if (power[me] + maxtown < 0)
      return power[me];
    else {
      source[me] = m;
      return power[me] + maxtown;
    }
  }
  assert(0);
}

int main(void) {
  fgets(buf, BUFLEN-1, stdin);
  sscanf(buf, "%d%d", &N, &power[1]);
  power[1] = -power[1]; // if town, the power is negative

  // read the input
  for (int i=2;i<=N;i++) {
    char type;
    fgets(buf, BUFLEN-1, stdin);
    sscanf(buf, "%c%d%d%d", &type, &parent[i], &capacity[i], &power[i]);

    if (type == 'M') // town
      power[i] = -power[i];

    if (type == 'E' && power[i] == 0) { // power station with no production, mark this specially
      capacity[i] = -1;
      parent[i] = 0;
    }

  }

  if (walk_tree(1) >= 0) {
    for (int i=1;i<=N;i++)
      printf("%d%c", get_source(i), i == N ? '\n' : ' ');
  } else
    fail();

  return 0;
}
